<object name="jmolApplet_%(rowid)s" id="jmolApplet_%(rowid)s" type="application/x-java-applet" width="100%%" height="100%%">
  <param name="progressbar" value="true" />
  <param name="progresscolor" value="blue" />
  <param name="boxbgcolor" value="white" />
  <param name="boxfgcolor" value="black" />
  <param name="boxmessage" value="Downloading JmolApplet ..." />
  <param name="archive" value="JmolApplet0.jar" />
  <param name="mayscript" value="true" />
  <param name="codebase" value="%(jmol_code_base)s" />
  <param name="code" value="JmolApplet" />
  <param name="script" value="load %(3dpath)s; %(jmol_script_cmds)s" />
</object>
